/**
 * @fileoverview 分屏代码，推迟加载img和iframe，以及加载更多
 * @version 1.0
 * @author lexli & jessezhang & popotang  $LastChangedBy: teeniechen$
 * @lastUpdate 2015-8-10
 */
define(function(require, exports, module) {
	var lazyLoad = (function(){

		var Live = Live || {};
		var map_el = {};
		var el_obj = [];
		var download_count = 0;
		var last_offset = -1;// 记录曾经翻到的最大高度
		var doc_el;
		var lazy_load_tag;
		var map_func = {};
		var tmp_count = 0;
		var is_finish = true;

		/**
		 * 初始化变量
		 * 
		 * @param {Array}
		 *          tags 需要分频的TagName，默认为["img","iframe"]
		 * @param {Object} 页面滚动的对象
		 */
		function initVar(tags, scrollElm) {
			if (scrollElm) {
				doc_el = scrollElm;
			}
			else {
				doc_el = document.compatMode == 'BackCompat' ? document.body : document.documentElement;
			}

			lazy_load_tag = tags || ["img", "iframe"];
		};

		/**
		 * 初始化获取推迟加载的对象 TODO 预防多次初始化，重复添加相同的DOM元素 - xingkaiwang
		 * 
		 */
		function initElementMap() {
			// 获取当次初始化的dom元素数组
			var all_el = [];
			for (var i = 0, len = lazy_load_tag.length; i < len; i++) {
				var $el = $(lazy_load_tag[i] + "[lz_src]"),
					el = $el.elements || $el.get();
				for (var j = 0, len2 = el.length; j < len2; j++) {
					all_el.push(el[j]);
				}
			}

			// 将新获取的元素保存到已初始化所有dom元素的数组
			var start = el_obj.length,
				end = start;// 记录新加入的元素的索引位置
			for (var key = 0, len = all_el.length; key < len; key++) {
				if (typeof(all_el[key]) == "object" && all_el[key].getAttribute("lz_src")) {
					el_obj.push(all_el[key]);
					end++;
				}
				else {
					// what is this
					// log("not an object:" + key + "-" + typeof(all_el[key])
					// + " lz_src:" + all_el[key].getAttribute("lz_src"));
				}
			}

			// 将新加入的元素的索引添加到坐标集合中
			for (var i = start; i < end; i++) {
				var o_img = el_obj[i];
				// o_img.style.visibility = "hidden";
				// var t_index = Math.round(QZFL.dom.getXY(o_img)[1] - QZFL.dom.getXY(doc_el)[1])// 元素的top要减掉当前滚动容器的top
				var t_index = Math.round($(o_img).offset().top - $(doc_el).offset().top);// 元素的top要减掉当前滚动容器的top
				if (map_el[t_index]) {
					map_el[t_index].push(i);
				}
				else {
					var t_array = [];
					t_array[0] = i;
					map_el[t_index] = t_array;
					download_count++;
				}
			}
		};

		/**
		 * 计算屏幕显示的高度，这是延时加载的核心方法，单独抽出来，可由调用方自定义
		 * @return {Number}
		 */
		function getScreenShowHeight() {
			// 兼容chrome和Safari下的滚动加载，WebKit内核的浏览器没有scrollTop属性，要用window.pageYOffset替代
			// -vicyao
			var scrollTop = window.pageYOffset || doc_el.scrollTop;
			// log("doc_el.clientHeight:" + doc_el.clientHeight + " scrollTop:" +
			// scrollTop);
			return doc_el.clientHeight + scrollTop;
		};

		/**
		 * 推迟加载监听
		 * 
		 */
		function doLoading() {
			tmp_count++;

			if (!download_count) {
				is_finish = true;
				return;
			}

			var img_show_height = getScreenShowHeight();

			if (last_offset >= img_show_height) {
				setTimeout(doLoading, 200);
				return;
			}

			last_offset = img_show_height;

			for (var key in map_el) {
				if (img_show_height > key) {
					var t_o = map_el[key],
						img_vl = t_o.length, tmp_src;
					for (var l = 0; l < img_vl; l++) {
						tmp_src = el_obj[t_o[l]].getAttribute("lz_src");
						// if (tmp_src) {//空lz_src也赋给src，以触发onerror事件
							el_obj[t_o[l]].src = tmp_src;
							el_obj[t_o[l]].removeAttribute("lz_src");
							el_obj[t_o[l]].style.visibility = "visible";
						// }
					}
					delete map_el[key];
					download_count--;
				}
				else {
					// log("taller:" + key);
				}
			}

			for (var key in map_func) {
				if (img_show_height > key) {
					var t_o = map_func[key],
						img_vl = t_o.length;
					for (var l = 0; l < img_vl; l++) {
						// function called
						t_o[l]();
					}
					delete map_func[key];
					download_count--;
				}
			}

			setTimeout(doLoading, 200);
		};

		/**
		 * 加载全部图片
		 */
		function appleShowImg() {
			var tmp_src;
			for (var i = 0, len = el_obj.length; i < len; i++) {
				tmp_src = el_obj[i].getAttribute("lz_src");
				// if (tmp_src) {//空lz_src也赋给src，以触发onerror事件
					el_obj[i].src = tmp_src;
					el_obj[i].removeAttribute("lz_src");
					el_obj[i].style.visibility = "visible";
				// }
			}
			delete el_obj;
			delete map_el;
			download_count = 0;
		}

		/**
		 * 封装对苹果终端的判断
		 * 
		 * @namespace Live.apple
		 * @type {Object}
		 */
		Live.apple = (function() {
			var _ipad = false, _ipod = false, _iphone = false, _ua = navigator.userAgent;
			var _ipadExp = /\(ipad;/i, _iphoneExp = /\(iphone;/i, _ipodExp = /\(ipod;/i;
			if (_ipadExp.test(_ua)) {
				_ipad = true;
			}
			else if (_iphoneExp.test(_ua)) {
				_iphone = true;
			}
			else if (_ipodExp.test(_ua)) {
				_ipod = true;
			}
			return {
				/**
				 * 是否是ipad
				 * 
				 * @field ipad
				 * @memberOf Live.apple
				 */
				ipad : _ipad,
				/**
				 * 是否是iphone
				 * 
				 * @field iphone
				 * @memberOf Live.apple
				 */
				iphone : _iphone,
				/**
				 * 是否是ipod touch
				 * 
				 * @field ipod
				 * @memberOf Live.apple
				 */
				ipod : _ipod,
				/**
				 * 是否是苹果公司的产品(iPad,iPhone,iPod Touch)
				 * 
				 * @field appleProduct
				 * @memberOf Live.apple
				 */
				appleProduct : (_ipad || _iphone || _ipod)
			}
		})();

		/**
		 * 初始化分屏
		 * 
		 * @param {Array}
		 *          tags 需要分频的TagName，默认为["img","iframe"]
		 * @example
		 * Live.lazyLoad.init();//不带参数，分屏img和iframe
		 * Live.lazyLoad.init(["img"]);//分屏img
		 * Live.lazyLoad.init(["iframe"]);//分屏iframe
		 *
		 * @param {Object} scrollElm 页面滚动的对象
		 */
		function init(tags, scrollElm) {
			initVar(tags, scrollElm);
			initElementMap();

			is_finish = false;
			// 初始化后必须手动执行一次doLoading函数，防止有些加载项已经暴露在可视范围内
			setTimeout(function() {
				    doLoading();
			    }, 0);
		};

		/**
		 * 延时加载函数，根据元素的高度来延时调用初始化函数
		 * 
		 * @param {Object} element 元素对象
		 *          延时加载的参照元素，会根据该元素所在的位置来判断何时调用函数
		 * @param {Function} callback 回调函数
		 * @param {Number} fix 高度偏移量
		 */
		function lazyFunc(element, callback, fix) {
			fix = fix || 0;
			if (!element.getXY) {
				element = $(element);
			}

			var t_index = element.offset().top + fix;

			// log("y=" + t_index);

			if (map_func[t_index]) {
				map_func[t_index].push(callback);
			}
			else {
				var t_array = [];
				t_array[0] = callback;
				map_func[t_index] = t_array;
				download_count++;
			}
			
			//下面这行代码是解决一个bug，当页面打开后马上滚动到下方，这时候如果部分lazyFunc事件还没插入，就会导致因为last_offset大于当前可视高度，而无法执行doloading。
			//解决办法就是每次插入新的事件后都重置一下	by vicyao 2013-11-28
			last_offset = 0;
			
			if (is_finish) {
				is_finish = false;
				doLoading();
			}
		};
		/**
		 * 重置上一次的偏移量
		 * @param  {Number} offset [偏移量]
		 */
		function resetLastOffset(offset) {
			last_offset = offset;
		};

		/**
		 * 重置各种属性
		 */
		function reset() {
			map_el = {};
			el_obj = [];
			download_count = 0;
			last_offset = -1;
			lazy_load_tag;
			map_func = {};
			tmp_count = 0;
			is_finish = true;
		}
		return {
	        init: init,
	        reset: reset
	    }
	})();

	/**
	 * 加载更多
	 */
	var lazyLoadMore = (function() {

	    var finish_offset = 10000; //反倒这个高度则触发回调函数
	    var last_offset = -1; //记录曾经翻到的最大高度
	    var doc_el;
	    var funcobj = {}; //回调函数
	    var tmp_count = 0;
	    var is_finish = true;
	    var timer; // 计时器

	    /**
	     * 初始化变量
	     *
	     * @param {Array}
	     *          tags 需要分频的TagName
	     */
	    function initVar(scrollElm) {
	        if (scrollElm) {
	            doc_el = scrollElm;
	        } else {
	            doc_el = document.compatMode == 'BackCompat' ? document.body : document.documentElement;
	        }
	    };

	    /**
	     * 计算屏幕显示的高度，这是延时加载的核心方法，单独抽出来，可由调用方自定义
	     */
	    function getScreenShowHeight() {
	        // 兼容chrome和Safari下的滚动加载，WebKit内核的浏览器没有scrollTop属性，要用window.pageYOffset替代 -vicyao
	        var scrollTop = window.pageYOffset || doc_el.scrollTop;
	        // log("doc_el.clientHeight:" + doc_el.clientHeight + " scrollTop:" + scrollTop);
	        return doc_el.clientHeight + scrollTop;
	    };

	    /**
	     * 推迟加载监听
	     *
	     */
	    function doLoading() {
	        tmp_count++;
	        var img_show_height = getScreenShowHeight();

	        if (last_offset >= img_show_height) {
	            timer = setTimeout(doLoading, 200);
	            return;
	        }

	        last_offset = img_show_height;
	        if (last_offset >= finish_offset) {
	            if (typeof funcobj.fun == "function") {
	                funcobj.fun.call();
	            }
	            is_finish = true;
	            return;
	        }

	        timer = setTimeout(doLoading, 200);
	    };

	    /**
	     * 初始化变量
	     *
	     * @param offset 触发事件的高度
	     *          fn 回调函数
	     */
	    function init(offset, fn, scrollElm) {
	        initVar(scrollElm);
	        last_offset = -1;
	        finish_offset = offset;
	        funcobj.fun = fn;

	        is_finish = false;
	        // 初始化后必须手动执行一次doLoading函数，防止有些加载项已经暴露在可视范围内
	        setTimeout(function() {
	            doLoading();
	        }, 0);
	    };

	    function morereset(offset, fn) { //重新定位最下端
	            last_offset = -1;
	            tmp_count = 0;
	            finish_offset = offset;
	            if (typeof fn == "function") {
	                funcobj.fun = fn;
	            }
	            if (is_finish) { //完成了就重新起一个循环
	                setTimeout(function() {
	                    doLoading();
	                }, 0);
	            }
	        }
	        /**
	         * 停止加载
	         */
	    function finish() {
	        clearTimeout(timer);
	        is_finish = true;
	    }

	    return {
	        init: init,
	        getScreenShowHeight: getScreenShowHeight,
	        reset: morereset,
	        finish: finish
	    }
	})();

	module.exports = {		
		init : lazyLoad.init,
		reset : lazyLoad.reset,
		more : lazyLoadMore
	}
});