define(function(require, exports, module) {
    var utils = require('../../utils');

    //重组aid
    var REBUILD_ORDER = "$2$4$5$12$13$1$7$11$15$6$3$8$9$10$14$21"; //最后拼接成aid的顺序
    function _getMap(aid) {
        var map = {},
            match,
            reg = /(\$\d+):[^$]*/g;
        while ((match = reg.exec(aid)) != null) {
            map[match[1]] = match[0];
        }
        return map;
    }

    function _rebuildAid(aid) {
        var map = _getMap(aid);
        return REBUILD_ORDER.replace(/\$\d+/g, function(k) {
            return map[k] || '';
        });
    }

    /**
     * 获取支付统计aid
     * @return {[type]} [description]
     */
    function _getAid(param, aidPass) {
        /*
         1:渠道  channel
         2:平台  plat
         3:版本号
         4:来源 source
         5:cid
         6:Vid
         7:Pid
         8:二级来源
         11:支付场景
         12:ptag
         13:活动
         15:连续包月 serial

         aid 按 2~4~5~12~13~1~7~11~15~6~3~8~9~10~ 进行排序组参
         分别对应：平台、一级来源、CID、ptag、活动号、（启动）渠道、直播PID，支付场景（指扫码或者MINIPAY）、产品包、VID、版本号、二级来源、频道ID、列表ID
         */
        var aid = '';
        var ptag = utils.getUrlParam('ptag') || utils.getUrlParam('PTAG') || utils.getUrlParam('adtag') || utils.getUrlParam('ADTAG');
        if (!ptag) {
            if (typeof param === 'string') {
                ptag = param;
            } else if (param && param.ptag) {
                ptag = param.ptag;
            }
        }
        if (ptag) {
            ptag = decodeURIComponent(ptag);
        }
        var aid_param = aidPass || decodeURIComponent(utils.getUrlParam('aid')) || "";
        var actid_param = utils.getUrlParam('hlwactid') || utils.getUrlParam('mpid');
        var cid = utils.getUrlParam('cid');
        if (param && param.cid) {
            cid = param.cid;
        }
        var vid = utils.getUrlParam('vid');
        if (param && param.vid) {
            vid = param.vid;
        }
        var actid = '';
        if (actid_param) {
            actid = actid_param;
        } else {
            if (param && param.actid) {
                actid = param.actid;
            }
        }
        var channel = (param && param.channel) ? param.channel : '';
        var plat = (param && param.plat) ? param.plat : utils.getPlatform();
        var source = (param && param.source) ? param.source : '';

        if (aid_param && aid_param.indexOf('V0$$') != -1) {
            var aid_txt = aid_param.split('$$')[1];
            if (aid_txt) {
                //去掉空的节点
                var aid_temp = [];
                var arr_aid = aid_txt.split('$');
                for (var i = 0; i < arr_aid.length; i++) {
                    var aid_item = arr_aid[i];
                    if (aid_item) {
                        var arr_part = aid_item.split(':');
                        if (arr_part.length > 0 && arr_part[1] != '') {
                            aid_temp.push(aid_item);
                        }
                    }
                };
                aid_param = ['$', aid_temp.join('$')].join('');
            }
            aid = aid_param;
            if (aid.indexOf('$1:') == -1 && channel) {
                aid += ['$1:', channel].join('');
            }
            if (aid.indexOf('$2:') == -1) {
                if (plat) {
                    aid += ['$2:', plat].join('');
                } else {
                    aid += ['$2:', utils.getPlatform()].join('');
                }
            }
            if (aid.indexOf('$4:') == -1 && source) {
                aid += ['$4:', source].join('');
            }
            if (aid.indexOf('$12:') == -1 && ptag) {
                aid += ['$12:', encodeURIComponent(ptag)].join('');
            }
            if (aid.indexOf('$13:') == -1 && actid) {
                aid += ['$13:', actid].join('');
            }
        } else {
            //aid = 'V0$';
            if (plat) {
                aid += ['$2:', plat].join('');
            } else {
                aid += ['$2:', utils.getPlatform()].join('');
            }
            if (source) {
                aid += ['$4:', source].join('');
            }
            if (cid) {
                aid += ['$5:', cid].join('');
            }
            if (ptag) {
                aid += ['$12:', encodeURIComponent(ptag)].join('');
            }
            if (actid) {
                aid += ['$13:', actid].join('');
            } else if (aid_param && aid_param != null) {
                aid += ['$13:', encodeURIComponent(aid_param)].join('');
            }
            if (channel) {
                aid += ['$1:', channel].join('');
            }
            if (vid) {
                aid += ['$6:', vid].join('');
            }
        }

        //rebuild aid
        var aidRebuild = _rebuildAid(_buildExtraParam(aid, param));
        aid = ['V0$', aidRebuild].join('');
        return aid;
    }

    function _buildExtraParam(aid, param) { //判断是否是连续包月，弹窗版（aphone）,绕iap(iphone)
        var serial = (param && param.isSerial) ? param.isSerial : ''; //是否期自动续费
        var isDialog = (param && param.isDialog) ? param.isDialog : ''; //是否是弹窗版
        var isUseH5 = (param && param.useH5) ? param.useH5 : ''; //是否是弹窗版
        var isPay = (param && param.isPay) ? param.isPay : ''; //是否是支付相关
        var couponId = (param && param.couponId) ? param.couponId : ''; //代金券groupid(cardid)
        if (utils.isIosPf()) {
            var buildVersion = (param && param.appInfo && param.appInfo.buildVersion) ? param.appInfo.buildVersion : '';
            var isJBOS = (param && param.appInfo && param.appInfo.isJBOS) ? param.appInfo.isJBOS : '';
            if (aid.indexOf('$3:') == -1 && buildVersion) {
                aid += ['$3:', buildVersion].join('');
            }
            if (aid.indexOf('$11:') == -1 && isPay) { //支付点击才上报第11级节点
                //第11级节点--iphone：越狱：4；绕iap：7;iap:14
                if (isJBOS == 1) {
                    aid += '$11:4';
                } else if (isUseH5 == 1) {
                    aid += '$11:7';
                } else if (isUseH5 == 2) {
                    aid += '$11:15';
                } else {
                    aid += '$11:14';
                }
            }
        } else {
            if (aid.indexOf('$11:') == -1 && isPay) { //支付点击才上报第11级节点
                //aid += '$11:6'(之前走h5的上报);
                //第11级节点--安卓：计平弹窗：12；计平渠道页：13
                if (isDialog) {
                    aid += '$11:12';
                } else {
                    aid += '$11:13';
                }
            }
        }

        if (aid.indexOf('$15:') == -1 && serial) { //是否是自动续费
            aid += ['$15:', serial].join('');
        }

        if (aid.indexOf('$21:') == -1 && couponId) { //代金券groupid(cardid)
            aid += ['$21:', couponId].join('');
        }
        return aid;

    }

    module.exports = {init:_getAid};
});
