/**
 *	@author: jackqqxu@tencent.com
 *  @modify: 2014-12-12
 *  @desc:  主要是用来实现自动轮播，随时停止轮播，手势滑动图片跟随，在很多H5页面的头部常见的轮播效果；
 *
 *	主要功能如下：
	a)  除滑动浏览外且支持自动循环轮播，从第一张轮播到最后一张，再继续循环轮播回第一张，以此类推。
	b)  当用户手动滑动浏览时，自动循环轮播暂停，优先响应用户手动操作。待用户手动操作停止后，重新继续轮播，但此时从用户滑动停止时定位的这张cover开始轮播。
	c)  当用户滑动按住某张图时，判断滑动是否超过一半，若滑动超过一半，取消点击时，自动切入下一张。若滑动没有超过一半时，取消点击时，应该定位在该cover图上。
    d)	作为一个zepto的插件存在，直接可以启动操作；
    e)  需要引入zepto，即如果作为模块化，则使用require('zepto')可以引用到zepto模块

    对DIV的结构的基本要求如下:

    <style>
        .ui-slide-banner{
            margin-right: -100%;
            float:left;
            position:relative;
        }
    </style>

    <div style="overflow: hidden;">
        <ul>
            <li class="ui-slide-banner" style="width: 100%;left:0">
                <img src="https://qzonestyle.gtimg.cn//qzone/space_item/cover/oper/2014-12-04/b_1417679660.jpg" style="width:100%;">
            </li>
            <li class="ui-slide-banner" style="width: 100%;left:100%">
                <img src="https://qzonestyle.gtimg.cn//qzone/space_item/cover/oper/2014-11-28/b_1417159220.jpg" style="width:100%;">
            </li>
            <li class="ui-slide-banner" style="width: 100%;left:100%">
                <img src="https://qzonestyle.gtimg.cn//qzone/space_item/cover/oper/2014-11-28/b_1417159262.jpg" style="width:100%;">
            </li>
            <li class="ui-slide-banner" style="width: 100%;left:100%">
                <img src="https://qzonestyle.gtimg.cn//qzone/space_item/cover/oper/2014-11-12/b_1415787125.jpg" style="width:100%;">
            </li>
        </ul>
     </div>

    调用示例如下：
    self.slideObj = new slide();
    self.slideObj.render($('#J_home .ui-slide-banner'), 5, 0, 400, function(i){
        callback(i);    //i表示当前第几张图片的索引值，范围从0到张数-1
    });
 */


;(function(global) {
    var slide = function () {
        this.clearId = null; //setInterval() 返回的 ID 值
        this.inited = false;
        this.i = 0;
        this.prevIndex = 0;
        this.nextIndex = 0;
        this.isDoubule = false;
    };


    //慢速平滑移动方法
    var slowTranslate = function(dom, width, during){
        dom.css({
            '-webkit-transform': 'translateZ(0px) translateX(' + width + ')',
            'transform': 'translateZ(0px) translateX(' + width + ')',
            '-webkit-transition' : 'all ' + during + 's ease',
            'transition' : 'all ' + during + 's ease'
        });
    }

    //快速定位动方法
    var fastTranslate = function(dom, width){
        dom.css({
            '-webkit-transform' : 'translateZ(0px) translateX(' + width + ')',
            'transform' : 'translateZ(0px) translateX(' + width + ')',
            '-webkit-transition' : 'all 0 ease',
            'transition' : 'all 0 ease'
        })
    }

    /**
     * @param slides   ul元素的id值 li元素的class名称，主要是用来搜索的，生成li的数组
     * @param timeGap   几秒钟运行滑动一次，如果为0，则不自动滑动
     * @param initIndex 初始化为多少，则默认为第一张图的索引0
     * @param timeInterval  多长时间滚动100%
     * @param callback  回调函数，左滑动和右滑动触发同样的回调函数，当前索引会作为参数传入
     */
    slide.prototype = {
        render: function (slides, timeGap, initIndex, timeInterval, callback) {
            var i = initIndex || this.i, startPos = 0, movePos = 0, endPos = 0, prevIndex = 0, nextIndex = 0, timeInterval = timeInterval || 500;
            var width = slides.width(), that = this, slideLen = slides.length, startTime = 0, endTime = 0, callback = callback || '';
            var isDouble = false;
            this.width = width;
            if (slideLen < 2) {
                return false;
            }
            if (slideLen == 2) {    //只有2个元素的时候，考虑复制一份
                var slideParent = slides.parent();
                var p = slides.clone();//.append($(slides).text());
                slides.parent().prepend(p);
                isDouble = true;
                slides = slideParent.find('li');
                fastTranslate($(slides[1]), '100%');
                fastTranslate($(slides[2]), '100%');
                fastTranslate($(slides[3]), '-100%');
                slideLen = 4;
            }
            if (that.inited) {
                that.autoExec(timeGap, slides, timeInterval, callback, i);
                return;
            }
            that.i = i;
            that.isDouble = isDouble;
            slides.unbind('touchstart').bind('touchstart', function (e) {
//                console.log('e=' + e.touches);
                startPos = e.touches[0].pageX;
                startTime = +new Date();
                if (!timeGap) {
                    clearInterval(that.clearId);
                    that.clearId = null;
                }
            }).bind('touchmove', function (e) {
                e.preventDefault();
                clearInterval(that.clearId);
                that.clearId = null;
                movePos = e.touches[0].pageX;
//                console.log('touchmove pos=' + movePos)
                var movePercent = Math.abs(movePos - startPos) / parseInt(width);
                if (movePos > startPos) {  //手指从左向右滑动
//                    console.log('movePos=' + movePos + '; startPos=' + startPos)
                    if (movePos > width) {
                        return;
                    }
                    that.prevIndex = that.i - 1;
//                    console.log('that.i =' + that.i );
                    if (that.prevIndex < 0) { //考虑滑动到第一张的时候
                        that.prevIndex = slideLen - 1;
                        fastTranslate($(slides[that.prevIndex]), '-' + width + 'px');
                    }
                    fastTranslate($(slides[that.prevIndex]), '-' + (width * (100 - movePercent * 100) / 100) + 'px');
                    fastTranslate($(slides[that.i]), (width * movePercent) + 'px');
                } else {    //手指从右向左滑动
                    if (movePos < 0) {
                        return;
                    }
                    that.nextIndex = that.i + 1;
                    if (that.nextIndex >= slideLen) {
                        that.nextIndex = 0;
                        fastTranslate($(slides[that.nextIndex]), width + 'px');
                    }
                    fastTranslate($(slides[that.nextIndex]), (100 - movePercent * 100) / 100 * width + 'px');
                    fastTranslate($(slides[that.i]), '-' + movePercent * width + 'px');
                }
            }).bind('touchend', function (e) {
                endPos = e.changedTouches[0].pageX;
                endTime = +new Date();
                var overHalfWidth = Math.abs(endPos - startPos) >= (width / 2);
                if(endTime - startTime < 200) { //看这里多少合适；相当于用户是快速在滑动，200毫秒
//                    console.log('timeGap=' + (endTime - startTime));
                    overHalfWidth = true;
                }
//                console.log('touchend pos=' + endPos + '; l=' + Math.abs(endPos - startPos) + ';')
				if (endPos == startPos) {
                    return;
                }
                if (endPos > startPos) {  //手指从左向右滑动
                    if (overHalfWidth) {
                        slowTranslate($(slides[that.i]), width + 'px', timeInterval / 1000);
                        slowTranslate($(slides[that.prevIndex]), '0px', timeInterval / 1000);
                        if (that.prevIndex - 1 >= 0) {
                            fastTranslate($(slides[that.prevIndex - 1]), '-' + width + 'px');
                        } else {
                            fastTranslate($(slides[slideLen - 1]), '-' + width + 'px');
                        }
                        that.i = that.prevIndex;
                        callback && callback(that.isDouble ? (that.i % 2) : that.i);
                    } else {
                        slowTranslate($(slides[that.prevIndex]), '-' + width + 'px', timeInterval / 1000);
                        slowTranslate($(slides[that.i]), '0px', timeInterval / 1000);
                    }
                } else {    //手指从右向左滑动
                    if (overHalfWidth) {
                        slowTranslate($(slides[that.i]), '-' + width + 'px', timeInterval / 1000);
                        slowTranslate($(slides[that.nextIndex]), '0px', timeInterval / 1000);
                        if (that.nextIndex + 1 >= slideLen) {
                            fastTranslate($(slides[0]), width + 'px');
                        } else {
                            fastTranslate($(slides[that.nextIndex + 1]), width + 'px');
                        }

                        that.i = that.nextIndex;
                        callback && callback(that.isDouble ? (that.i % 2) : that.i);
                    } else {
                        slowTranslate($(slides[that.nextIndex]), width + 'px', timeInterval / 1000);
                        slowTranslate($(slides[that.i]), '0px', timeInterval / 1000);
                    }
                }
                that.autoExec(timeGap, slides, timeInterval, callback, that.i);
            });
            this.autoExec(timeGap, slides, timeInterval, callback, that.i);
            this.inited = true;
            this.length = slides.length;
        },

        autoExec: function (timeGap, slides, timeInterval, callback, index) {
            var self = this;//return;
            self.i = typeof index != 'undefined' ? index : 0;
//            console.log('self.clearId=' + self.clearId + '; timeGap=' + timeGap);//return;
            if (timeGap) {
                if (self.clearId) {
                    return;
                }
                if (slides.length < 2) {
                    return false;
                }
                self.clearId = setInterval(function () {
                    self.nextIndex = self.i + 1;
                    if (self.i + 1 >= self.length) {
                        self.nextIndex = 0;
                        self.i = slides.length - 1;
                    }
//                    console.error('autoExec math=' + Math.random() + 'this.clearId=' + self.clearId);
//                    console.log('over half; i=' + self.i + '; nextIndex=' + self.nextIndex + '; timeInterval=' + timeInterval + '; ' + Math.random() + '; callback=' + callback);
                    slowTranslate($(slides[self.i]), '-' + self.width + 'px', timeInterval / 1000);
                    slowTranslate($(slides[self.nextIndex]), '0px', timeInterval / 1000);
                    if (self.nextIndex + 1 >= self.length) {
                        fastTranslate($(slides[0]), self.width + 'px');
                    } else {
                        fastTranslate($(slides[self.nextIndex + 1]), self.width + 'px');
                    }
                    self.i++;
                    if (self.i >= self.length) {
                        self.i = 0;
                    }
                    callback && callback(self.isDouble ? (self.i % 2) : self.i);
                }, timeGap * 1000);
            }
        },

        //用来停止自动切换
        stop : function(){
            var self = this;
            clearInterval(self.clearId);
            self.clearId = null;
        }
    }

    if (typeof define === 'function') {
        define(function(){
            return slide;
        })
    } else {
        global.slide = slide;
    }
})(window);
