
/**
 *@author   tonicli
 *@date     2016/6/5
 */
define(function (require, exports, module) {
'use strict';
function _toString(v) {
    return Object.prototype.toString.call(v);
}
function _isObject(v) {
    return _toString(v) == '[object Object]';
}
function _trim(v) {
    if (_toString(v) != '[object String]') {
        return v;
    }
    return v.replace(/^\s+|\s+$/g, '');
}
function _buildUrl(url, params, useEncode) {
    var queryStr = '';

    if (!url) {
        throw new Error('[util.url.buildUrl]url is required');
    }
    if (!params || params === null) {
        return url;
    }
    queryStr = _buildParams(params, useEncode);
    if (!queryStr || !queryStr.length) {
        return url;
    }
    return [url , (url.indexOf('?') > -1 ? '&' : '?') , queryStr].join('');
}
function _buildParams(params, useEncode) {
    var k, v, query, r = [];
    for (k in params) {
        v = params[k];
        query = _buildParam(k, v, useEncode);
        if (query != null && query !== '') {
            r.push(query);
        }
    }
    return r.join('&');
}
function _buildParam(k, v, useEncode) {
    var type = typeof v,
        prop,
        i = 0,
        len,
        r = [];

    if (v === '') {
        return '';
    }
    if (type == 'number' || type == 'string' || type == 'boolean') {
        return [k, useEncode ? encodeURIComponent(v) : v].join('=');
    }
    if (_isObject(v)) {
        for (prop in v) {
            r.push(_buildParam([k, '[', prop, ']'].join(''), v[prop], useEncode));
        }
    } else if (Array.isArray(v)) {
        for (len=v.length; i<len; i++) {
            r.push(_buildParam([k, '[', i, ']'].join(''), v[i], useEncode));
        }
    }
    //过滤null, undefined, ''
    r = r.filter(function (item) {
        return (item != null && _trim(item) !== '');
    });
    return r.length ? r.join('&') : '';
}
/**
 *@desc 将url格式化为标准格式
 *@param url
 *@param [useRelative]  是否转为相对协议，默认为true
 *@param [protocol]     强制转为指定协议，如果useRelative为false，并且未指定protocol，则取当前页面的protocol
 */
function _normalize(url, useRelative, protocol) {
    var absUrl = _abs(url);

    if (!/^(http:|https:)/.test(absUrl)) {
        return url;
    }
    if (typeof useRelative == 'undefined') {
        useRelative = true;
    }
    if (arguments.length == 2 && typeof useRelative == 'string' && useRelative.length > 0) {
        protocol = useRelative;
        useRelative = false;
    }
    if (useRelative) {
        return absUrl.replace(/^(http:|https:)/, '');
    }
    if (!protocol) {
        protocol = location.protocol;
    }
    if (!/:$/.test(protocol)) {
        protocol += ':';
    }
    return absUrl.replace(/^\w+:/, protocol);
}
function _abs(url) {
    var el = document.createElement('a');
    el.href = url;
    return el.href;
}
function _getHost(url) {
    var absUrl = _abs(url),
        match,
        oUrl,
        host = '';

    if (typeof URL !== 'undefined') {
        try {
            oUrl = new URL(absUrl);
            host = oUrl.host;
        } catch (e) {}
    }
    if (!host) {
        match = url.match(/^[\w^_]+:\/\/([\w\.\-]+)/);
        host = match && match.length > 1 ? match[1] : '';
    }

    return host;
}

module.exports = {
    buildUrl: _buildUrl,
    buildQueryStr: _buildParams,
    abs: _abs,
    getHost: _getHost,
    normalize: _normalize
};
})
