define(function(require, exports, module) {
    var storage = window.localStorage;
    var ua = navigator.userAgent;
    var isIos = ua.indexOf("iPhone") != -1 ? true : false;
    var isAndroid = ua.indexOf("Android") != -1 ? true : false;
    var appVersion = getAppVersion();
    var normalizer = require('./common/urlFactory').createNormalizer();

    /*获取后台返回的offerid
     * offeridParam：offerid1|appversion|offerid2|appversion...
     */
    function getOfferId(offeridParam) {
        var offerIdStr = offeridParam;
        var offeridVersionArr = offerIdStr.split('|');
        var groupLength = offeridVersionArr.length;
        var versionArr = new Array();
        if (groupLength > 0) {
            for (var i = 0; i < groupLength; i++) {
                if (i % 2 != 0) {
                    versionArr.push(offeridVersionArr[i]);
                }
            }

            if (versionArr.length > 0) {
                var rtnIndex = find(versionArr, appVersion);
                var realIndex = (rtnIndex + 1) * 2 - 1;
                if (realIndex >= 1 && realIndex <= groupLength - 1) {
                    var hitVersion = offeridVersionArr[realIndex];
                    if (appVersion >= hitVersion) {
                        offerIdStr = offeridVersionArr[realIndex + 1];
                    } else {
                        offerIdStr = offeridVersionArr[realIndex - 1];
                    }
                }
            }
        }
        return offerIdStr;
    }

    function find(array, val) {
        //如果值小于数组中最小的值时，则返回0
        if (val < Math.min.apply(null, array)) {
            return 0;
        };
        //如果值大于数组中最大的值时，则返回最大值
        if (val > Math.max.apply(null, array)) {
            return array.length - 1;
        };
        var idx = 0,
            i = 0,
            j = array.length;
        for (; i < j; i++) {
            if (array[i] > val) {
                idx = i;
                break;
            };
        };
        return idx;
    };

    //获取app版本号
    function getAppVersion() {
        var versionCode = "4.9.0";
        var versionCodeStr = ua.match(/QQLiveBrowser\/(\d+\.\d+.\d+)/);
        if (versionCodeStr != null) {
            versionCode = versionCodeStr[1];
        }
        return versionCode;
    }

    //获取app build版本号
    function getAppVersionBuild() {
        var versionCode = 8400;
        if (/ QQLiveBrowser\//.test(navigator.userAgent)) {
            var posBrowser = ua.indexOf("QQLiveBrowser");
            var tmp = ua.substring(posBrowser);
            var arrRtn = tmp.split('/');
            if (arrRtn.length >= 2) {
                var verTmp = arrRtn[1];
                var verTmpArr = verTmp.split('.');
                if (verTmpArr.length >= 4) {
                    versionCode = verTmpArr[3];
                }
            }
        }
        return versionCode;
    }

    //版本号比较X.X.X
    function versionCompare(version1, version2) {
        var v1Array = version1.split('.');
        var v2Array = version2.split('.');
        var result = false;
        if (v1Array.length == v2Array.length && v1Array.length > 2) {
            for (var i = 0; i < v1Array.length; i++) {
                var item1 = v1Array[i];
                var item2 = v2Array[i];
                var num1 = parseInt(item1, 10);
                var num2 = parseInt(item2, 10);
                if (num1 > num2) {
                    result = true;
                    break;
                } else if (num1 < num2) {
                    result = false;
                    break;
                } else {
                    result = true;
                }
            }
        }
        return result;
    }

    //获取价格配置接口
    function getHlwPriceConfig(hlw_param, suCb, failCb) {
        $.ajax({
            url: [normalizer("http://pay.video.qq.com/fcgi-bin/price-month?otype=json&hlw_params="), encodeURIComponent(hlw_param)].join(''),
            dataType: "jsonp",
            success: function(json) {
                if (json.result && json.result.code == 0) {
                    storage.setItem("app_hlw_user_discounts_new", JSON.stringify(json));
                    suCb && suCb(json);
                } else {
                    failCb && failCb();
                }
            },
            error: function(XMLHttpRequest, textStatus, errorThrown) {
                failCb && failCb();
            }
        });
    }

    //获取默认价格配置
    function getDefaultPrice(callback) {
        try {
            $.getJSON("./js/price_default.json", function(data) {
                callback && callback(data);
            });
        } catch (ex) {}
    }

    function time33(skey) {
        // 哈希time33算法
        for (var i = 0, len = skey.length, hash = 5381; i < len; ++i) {
            hash += (hash << 5) + skey.charAt(i).charCodeAt();
        };
        return hash & 0x7fffffff;
    }

    function getToken(skey) {
        return time33(skey);
    }


    //新版获取价格配置接口
    function getHlwDiscountAjax(pay_type, suCb, failCb) {
        var isInitDiscount = false;
        var skey = "";
        var platform = 7;
        var pay_typeParam = "";

        if (isAndroid) {
            platform = 8;
        }
        var hlw_param = ['pf=', platform, "&app=qqlive&appver=", appVersion].join('');
        var url = [normalizer("http://pay.video.qq.com/fcgi-bin/get_price_voucher?otype=json&hlw_params="), encodeURIComponent(hlw_param)].join('');
        if (pay_type) {
            url += '&pay_type=' + pay_type;
        }
        getGtk(function(g_tk) {
            url += "&g_tk=" + g_tk;
            $.ajax({
                url: url,
                dataType: "jsonp",
                //timeout: 2000,
                success: function(json) {
                    /*$('.app_container').show();
                    $('.g_container_loading').hide();*/ //价格接口返回展示页面
                    if (json.result && json.result.code == 0) {
                        isInitDiscount = true;
                        storage.setItem("app_hlw_user_discounts_conf", JSON.stringify(json));
                        //第二个参数标识价格是否是读取默认价格
                        suCb && suCb(json, false);
                    } else {
                        if (storage.getItem("app_hlw_user_discounts_conf")) {
                            var discounts = JSON.parse(storage.getItem("app_hlw_user_discounts_conf"));
                            isInitDiscount = true;
                            //第二个参数标识价格是否默认价格
                            suCb && suCb(discounts, true);
                        }
                        if (!isInitDiscount) {
                            failCb && failCb();
                            isInitDiscount = true;
                        }
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    $('.app_container').show();
                    $('.g_container_loading').hide(); //价格接口返回展示页面
                    if (storage.getItem("app_hlw_user_discounts_conf")) {
                        var discounts = JSON.parse(storage.getItem("app_hlw_user_discounts_conf"));
                        isInitDiscount = true;
                        //第二个参数标识价格是否是从缓存读取
                        suCb && suCb(discounts, true);
                    }
                    if (!isInitDiscount) {
                        failCb && failCb();
                        isInitDiscount = true;
                    }
                }
            });
        });
    }

    function getHlwDiscountDefault(useH5, callback) {
        var conf_path = "./js/price_default_h5.json";
        if (!useH5) {
            conf_path = "./js/price_default_iap.json";
        }
        try {
            $.getJSON(conf_path, function(data) {
                callback && callback(data);
            });
        } catch (ex) {}
    }

    function getDefaultHlwParam() {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        var hlw_param = ['pf=', platform, "&app=qqlive&appver=", appVersion].join('');
        return hlw_param;
    }

    //获取代金券数量请求
    function getCouponCountAjax(suCb, failCb) {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        var url = [normalizer("http://pay.video.qq.com/fcgi-bin/vip_vouchers?cmd=60563&vouchers_type=1&page_index=0&page_count=1&platform1="), platform, "&appver=", appVersion, "&otype=json&pf=wechat_wx-2001-wx-2011-hlw"].join('');
        getGtk(function(g_tk) {
            url += "&g_tk=" + g_tk;
            $.ajax({
                url: url,
                dataType: "jsonp",
                success: function(json) {
                    if (json.result && json.result.code == 0) {
                        suCb && suCb(json);
                    } else {
                        failCb && failCb();
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    failCb && failCb();
                }
            });
        });
    }

    //获取每个套餐可用代金券信息
    function getCouponPackageAjax(packPrice, suCb, failCb) {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        var url = [normalizer("http://pay.video.qq.com/fcgi-bin/vip_vouchers?cmd=60564&price_list="), packPrice, "&platform1=", platform, "&appver=", appVersion, "&otype=json&pf=wechat_wx-2001-wx-2011-hlw"].join('');
        getGtk(function(g_tk) {
            url += "&g_tk=" + g_tk;
            $.ajax({
                url: url,
                dataType: "jsonp",
                success: function(json) {
                    if (json.result && json.result.code == 0) {
                        suCb && suCb(json);
                    } else {
                        failCb && failCb();
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    failCb && failCb();
                }
            });
        });
    }

    function tenvideoReady(fun) {
        fun = fun || function() {};
        if (typeof TenvideoJSBridge == "undefined") {
            if (document.addEventListener) {
                document.addEventListener('onTenvideoJSBridgeReady', fun, false);
            } else if (document.attachEvent) {
                document.attachEvent('onTenvideoJSBridgeReady', fun);
            }
        } else {
            fun();
        }
    }

    function getMainCookie(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getMainCookie", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    suCb && suCb(json.result);
                } else {
                    failCb && failCb();
                }
            });
        });
    }

    function getCookieParam(cookieStr, key) {
        var cookieObj = {},
            cookieSplit = [],
            // 以分号（;）分组
            cookieArr = cookieStr.split(";");
        for (var i = 0, len = cookieArr.length; i < len; i++) {
            if (cookieArr[i]) {
                // 以等号（=）分组
                cookieSplit = cookieArr[i].split("=");
                // Trim() 是自定义的函数，用来删除字符串两边的空格
                cookieObj[cookieSplit[0]] = cookieSplit[1];
            }
        }
        return cookieObj[key];
    }

    function getAppInfo(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getAppInfo", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    storage.setItem("app_info", JSON.stringify(json.result));
                    suCb && suCb(json.result);
                } else {
                    failCb && failCb();
                }
            });
        });
    }

    function getIapType(suCb, failCb) {
        tenvideoReady(function() {
            var isCallback = false;
            /*if (storage.getItem("app_iap_type")) {
                var iapInfo = JSON.parse(storage.getItem("app_iap_type"));
                isCallback = true;
                suCb && suCb(iapInfo);
            }*/
            var d_app = $.Deferred();
            var d_cookie = $.Deferred();

            $.when(d_app, d_cookie).done(function(json_app, json_cookie) {
                var aid = getUrlParam('aid');
                if (!aid) {
                    aid = ['V0$$1:0$2:7$3:', json_app.buildVersion, '$11:6'].join('');
                }
                var loginParam = '';
                if (json_cookie) {
                    var accountType = json_cookie.type;
                    var str_cookie = json_cookie.cookie;
                    if ("qq" == accountType) {
                        var uin = getCookieParam(str_cookie, "uin");
                        loginParam = ['&uin=', uin].join('');
                    } else if ("wx" == accountType) {
                        var openid = getCookieParam(str_cookie, "openid");
                        var appid = getCookieParam(str_cookie, "appid");
                        loginParam = ['&openid=', openid, "&appid=", appid].join('');
                    }
                }
                var vertype = "1"
                if (json_app) {
                    if (json_app.isinreview == 1) {
                        vertype = 1;
                    } else {
                        vertype = 0;
                    }
                }
                var url = [normalizer('http://pay.video.qq.com/fcgi-bin/iaptype?otype=json'), loginParam, '&vertype=', vertype, '&aid=', aid].join('');
                $.ajax({
                    url: url,
                    dataType: "jsonp",
                    success: function(json) {
                        if (json && json.result.code == 0) {
                            storage.setItem("app_iap_type", JSON.stringify(json));
                            !isCallback && suCb && suCb(json);
                        } else {
                            !isCallback && failCb && failCb();
                        }
                    },
                    error: function(XMLHttpRequest, textStatus, errorThrown) {
                        !isCallback && failCb && failCb();
                    }
                });
            });

            getAppInfo(function(json) {
                d_app.resolve(json);
            }, function() {
                d_app.resolve();
            });

            getMainCookie(function(json) {
                d_cookie.resolve(json);
            }, function() {
                d_cookie.resolve();
            });
        });
    }

    function getMainUserInfo(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getMainUserInfo", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    suCb && suCb(json.result);
                } else {
                    failCb && failCb();
                }
            });
        });
    }

    function isAccountLogin(suCb, failCb) {
        getMainUserInfo(function(json) {
            if (json && json.userInfo) {
                suCb && suCb(json);
            } else {
                failCb && failCb();
            }
        }, function() {
            failCb && failCb();
        });
    }

    function getGtk(suCb) {
        var skey = "";
        var g_tk = "";
        getMainCookie(function(json_msg) {
            if (json_msg) {
                var str_cookie = json_msg.cookie;
                if (json_msg.type == 'qq') {
                    skey = getCookieParam(str_cookie, "skey");
                }
                g_tk = getToken(skey);
                suCb && suCb(g_tk);
            }
        }, function() {
            g_tk = getToken(skey);
            suCb && suCb(g_tk);
        });
    }

    function getTicketNum(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getTicketNum", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    var num = json.result.num;
                    suCb && suCb(num);
                } else {
                    failCb && failCb();
                }
            });
        });

    }

    function getOrderNum(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getOrderNum", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    var num = json.result.num;
                    suCb && suCb(num);
                } else {
                    failCb && failCb();
                }
            });
        });
    }

    function getOrderNumNetwork(suCb, failCb, isVVip) { //网络获取购买记录
        getMainCookie(function(json_user) {
            var cookieStr = json_user.cookie;
            var accountType = json_user.type;
            var uin = "";
            var skey = "";
            var g_tk = "";
            if (accountType == "qq") {
                uin = getUin(getCookieParam(cookieStr, "uin"));
                skey = getCookieParam(cookieStr, "skey");
                g_tk = getToken(skey);
            } else {
                g_tk = getToken(skey);
            }
            var url = "";
            var cgiUrl = normalizer("http://pay.video.qq.com/fcgi-bin/order?otype=json&t=1&psize=1&pnum=1");
            var platform = 7;
            if (isAndroid) {
                platform = 8;
            }
            url = [cgiUrl, "&platform=", platform].join('');
            if (platform == 8) {
                url += "&versioncode=" + getAppVersionBuild();
            }
            if (platform == 7) {
                if (isVVip == "true") {
                    url += "&isVisitor=1";
                }
            }
            var userInfo = "";
            userInfo = ["&g_tk=", g_tk, "&uin=", uin].join('');
            url += userInfo;
            getDeviceInfo(function(json) {
                var guid = json.guid;
                url += "&guid=" + guid;
                _orderAjax(url, suCb, failCb);
            }, function() {
                _orderAjax(url, suCb, failCb);
            });
        });
    }

    function _orderAjax(url, sucCb, failCb) {
        $.ajax({
            url: url,
            dataType: "jsonp",
            success: function(json) {
                if (json && json.ret == 0) {
                    sucCb && sucCb(json);
                } else {
                    failCb && failCb();
                }
            },
            error: function() {
                failCb && failCb();
            }
        });
    }

    function refreshVipInfo(sucCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("refreshVipInfo", null, function(json_msg) {
                sucCb && sucCb();
                //会触发调用reinits，刷新页面
                var json = JSON.parse(json_msg);
            });
        });
    }

    //获取是否是审核版本
    function isInreviewVer(callback) {
        if (isIos) {
            tenvideoReady(function() {
                TenvideoJSBridge.invoke("getAppInfo", null, function(json_msg) {
                    var json = JSON.parse(json_msg);
                    var vertype = 0;
                    if (json.errCode == 0) {
                        if (json.result.isinreview == 1) {
                            vertype = 1;
                        } else {
                            vertype = 0;
                        }
                        callback && callback(vertype);
                    } else { //失败了按审核版本走
                        vertype = 1;
                        callback && callback(vertype);
                    }
                });
            });

        } else {
            callback && callback();
        }
    }

    function getPlatform() {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        return platform;
    }

    function isAndroidPf() {
        return isAndroid;
    }

    function isIosPf() {
        return isIos;
    }

    function getUin(uinStr) {
        var ret = "";
        if (uinStr) {
            var reg = /^o0*/g;
            if (reg.test(uinStr)) {
                ret = parseInt(uinStr.replace(/^o0*/g, ""), 10);
            } else {
                ret = uinStr;
            }
        }
        return ret;
    }

    function getUserid(suCb) { //根据用户id缓存信息
        getMainCookie(function(json_user) {
            var cookieStr = json_user.cookie;
            var accountType = json_user.type;
            var uid = "";
            if (json_user.type == 'wx') {
                uid = getCookieParam(cookieStr, "openid");
            } else if (json_user.type == 'qq') {
                uid = getCookieParam(cookieStr, "uin");
                uid = getUin(uid);
            }
            if (uid) {
                suCb && suCb(uid);
            }
        });
    }

    function filterXSS(str) {
        return str.replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\"/g, "&quot;").replace(/\'/g, "&apos;");
    }

    //获取url中的参数
    function getUrlParam(name) {
        var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)"); //构造一个含有目标参数的正则表达式对象
        var r = window.location.search.substr(1).match(reg); //匹配目标参数
        if (r != null) return unescape(r[2]);
        return null; //返回参数值
    }

    function getDeviceInfo(suCb, failCb) {
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getDeviceInfo", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    suCb && suCb(json.result);
                } else {
                    failCb && failCb();
                }
            });
        });

    }

    //显示站内信入口:param：unreadNum(未读信息数),url(站内信详情页),show(是否显示。当隐藏时，不用传前两个参数)
    function showMessageButton(param, suCb, failCb) {
        if (versionCompare(appVersion, "5.0.1")) {
            tenvideoReady(function() {
                TenvideoJSBridge.invoke("showMessageButton", param, function(json_msg) {
                    var json = JSON.parse(json_msg);
                    if (json.errCode == 0) {
                        suCb && suCb(json.result);
                    } else {
                        failCb && failCb();
                    }
                });
            });
        }
    }

    function _getAbsoluteUrl(url) {
        var div = document.createElement('div');
        div.innerHTML = ['<a href=', url, '></a>'].join('');
        return div.firstChild.href;
    }

    function openUrlCommon(url) {
        tenvideoReady(function() {
            var absUrl = _getAbsoluteUrl(url);
            try {
                if (/^(http|https):\/\//.test(absUrl)) {
                    absUrl = 'txvideo://v.qq.com/Html5Activity?url=' + encodeURIComponent(absUrl);
                }
                setTimeout(function() {
                    TenvideoJSBridge.invoke('openView', {
                        url: absUrl
                    }, function(json_msg) {});
                }, 0);
            } catch (e) {
                location.href = url;
            }
        });
    }

    //获取是否自动续费开关
    function getAutoPayParam() {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        var $defer = $.Deferred();
        $.ajax({
            url: normalizer("http://pay.video.qq.com/fcgi-bin/get_config?name=458019&platform=") + platform + "&otype=json",
            dataType: 'jsonp',
        }).done(function(json) {
            if (json.result && json.result.code == 0 && json.isautopay == 1) {
                $defer.resolve(1);
            } else {
                $defer.resolve(0);
            }
        }).fail(function() {
            $defer.reject();
        });

        return $defer.promise();
    }

    function getSerialPayConf(is_useH5) {
        if (isIos) {
            if (versionCompare(appVersion, "5.4.0")) {
                return _getSerialPayConfWhiteList().then(function(res) {
                    var _has_auth = false;
                    if (res && res.ret == 0) {
                        if (res.has_auth) {
                            _has_auth = true;
                        } else {
                            _has_auth = false;
                        }
                    } else {
                        _has_auth = false;
                    }
                    if (_has_auth) {
                        return _isInreviewVerPromise().then(function(verType) {
                            if (verType == 1) {
                                $('.js_bind_zhifubao').hide();
                                return _getSerialPayConfCallbckInreview();
                            } else {
                                return _getSerialPayConfCallbck(is_useH5);
                            }
                        });
                    } else {
                        $('.js_proto_common').show();
                        $('.js_bind_zhifubao').hide();
                        $('.js_proto_serial').hide();
                        return $.Deferred().reject({ hasAuth: false });
                    }
                });
            } else {
                $('.js_proto_common').show();
                $('.js_bind_zhifubao').hide();
                $('.js_proto_serial').hide();
                return $.Deferred().reject({ hasAuth: false });
            }

            /*$('.js_proto_common').show();
            $('.js_proto_serial').hide();
            return $.Deferred().reject({hasAuth:false});*/
        } else {
            $('.js_proto_common').show();
            $('.js_bind_zhifubao').hide();
            $('.js_proto_serial').hide();
            return _getSerialPayConfCallbck(is_useH5);
        }
    }

    function _getSerialPayConfWhiteList() {
        var url = normalizer('http://vip.video.qq.com/fcgi-bin/comm_cgi?otype=json&name=white_list2&rule=cin_iap&cmd=1&otype=json');
        var $defer = $.Deferred();
        $.ajax({
            url: url,
            dataType: 'jsonp'
        }).done(function(res) {
            $defer.resolve(res);
        }).fail(function(err) {
            $defer.reject(err);
        });
        return $defer.promise();
    }

    function _getSerialPayConfCallbck(is_useH5) {
        var url = normalizer('http://mobile.video.qq.com/fcgi-bin/dout_iphone?otype=json&auto_id=1314&platform=7&version=0');
        //var url = normalizer('http://mobile.video.qq.com/fcgi-bin/dout_iphone?otype=json&auto_id=1315&platform=7&version=0');
        var $defer = $.Deferred();
        $.ajax({
            url: url,
            dataType: 'jsonp'
        }).done(function(res) {
            if (res && res.returncode == 0) {
                if (isIos && !is_useH5) {
                    var is_proto_serial_show = false;
                    for (var i = 0; i < res.serialPay_iphone.length; i++) {
                        var item = res.serialPay_iphone[i];
                        if (item.enable == 1) {
                            is_proto_serial_show = true;
                        }
                        break;
                    }
                    if (is_proto_serial_show > 0) {
                        $('.js_proto_common').hide();
                        $('.js_bind_zhifubao').show();
                        $('.js_proto_serial').show();
                    } else {
                        $('.js_proto_common').show();
                        $('.js_bind_zhifubao').hide();
                        $('.js_proto_serial').hide();
                    }
                    $defer.resolve(res.serialPay_iphone);
                } else {
                    $defer.resolve(res.serialPay_aphone);
                }
            } else {
                $defer.reject(res);
            }
        }).fail(function(err) {
            $defer.reject(err);
        });
        return $defer.promise();
    }

    function _getSerialPayConfCallbckInreview() {
        var url = normalizer('http://mobile.video.qq.com/fcgi-bin/dout_iphone?otype=json&auto_id=1314&platform=7&version=0');
        //var url = normalizer('http://mobile.video.qq.com/fcgi-bin/dout_iphone?otype=json&auto_id=1315&platform=7&version=0');
        var $defer = $.Deferred();
        $.ajax({
            url: url,
            dataType: 'jsonp'
        }).done(function(res) {
            if (res && res.returncode == 0) {
                var is_proto_serial_show = false;
                for (var i = 0; i < res.serialPay_iphone_inreview.length; i++) {
                    var item = res.serialPay_iphone_inreview[i];
                    if (item.enable == 1) {
                        is_proto_serial_show = true;
                    }
                    break;
                }
                if (is_proto_serial_show > 0) {
                    $('.js_proto_common').hide();
                    $('.js_bind_zhifubao').hide();
                    $('.js_proto_serial').show();
                } else {
                    $('.js_proto_common').show();
                    $('.js_bind_zhifubao').hide();
                    $('.js_proto_serial').hide();
                }
                $defer.resolve(res.serialPay_iphone_inreview);
            } else {
                $defer.reject(res);
            }
        }).fail(function(err) {
            $defer.reject(err);
        });
        return $defer.promise();
    }

    function _isInreviewVerPromise() {
        var $defer = $.Deferred();
        if (isIos) {
            tenvideoReady(function() {
                TenvideoJSBridge.invoke("getAppInfo", null, function(json_msg) {
                    var json = JSON.parse(json_msg);
                    var vertype = 0;
                    if (json.errCode == 0) {
                        if (json.result.isinreview == 1) {
                            vertype = 1;
                        } else {
                            vertype = 0;
                        }
                        $defer.resolve(vertype);
                    } else { //失败了按审核版本走
                        vertype = 1;
                        $defer.resolve(vertype);
                    }
                });
            });

        } else {
            var vertype = 0;
            $defer.resolve(vertype);
        }
        return $defer.promise();
    }

    //取专辑竖图小尺寸地址
    function picSizeSet(url, size) {
        // puui.qpic.cn域名图片规则
        //http://tapd.oa.com/MediaContent/markdown_wikis/#1010093801005051145
        var sizeArr = [75, 92, 116, 130, 161, 175, 220, 260, 350, 450, 770];
        if (sizeArr.indexOf(size) < 0) {
            size = 350;
        }
        return url.replace(/\/0$/, '/' + size);
    }

    //获取角标,新版角标
    function getCoverCorner(tag, tag2Tpl) {
        function format(tpl, data) {
            return tpl.replace(/\{(\w+)\}/g, function(m, w) {
                var val = (data[w] || '');
                if (val.indexOf("=") != -1) {
                    var varArr = val.split("=");
                    if (varArr.length > 0 && varArr[1]) {
                        val = varArr[1];
                    }
                }
                return normalizer(val);
            })
        }

        function legalParam(paramText) {
            if ((/\.(\w+)$/.test(paramText))) {
                return true;
            }
            return false;
        }

        function indexOf(item, arr) {
            for (var i = 0; i < arr.length; i++) {
                if (item === arr[i]) return i
            }
            return -1
        }

        var str = '',
            tag2TplDefault = '<i class="mark_v"><img src="{param2}" alt="{text}"></i>';
        tag2Tpl = tag2Tpl || tag2TplDefault;

        var positions = [2];

        if (tag) {
            // 先对tag进行处理，若为string类型的json则转换
            if (typeof tag === "string") {
                try {
                    tag = JSON.parse(tag);
                } catch (e) {
                    /* do nothing */
                }
            }
            // 出品类型：独播，腾讯出版
            if (tag.tag_2 && tag.tag_2.id > 0 && indexOf(2, positions) != -1) {
                if (legalParam(tag.tag_2["param2"])) {
                    str += format(tag2Tpl, tag.tag_2);
                }
            }
        }
        return str;
    }

    //获取类url中的参数形式的参数，如：a=xxx&b=xxx&c=xxx某个参数的值
    function getParam(paramsStr, name) {
        var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)"); //构造一个含有目标参数的正则表达式对象
        var r = paramsStr.match(reg); //匹配目标参数
        if (r != null) return unescape(r[2]);
        return null; //返回参数值
    }

    function getUrlParam2(url, name) {
        var reg = new RegExp('(\\?|&)' + name + '=([^&?]*)', 'i');
        var arr = url.match(reg);
        if (arr) {
            return arr[2];
        }
        return null;
    }

    function getAutoPayTag() {
        var platform = 7;
        if (isAndroid) {
            platform = 8;
        }
        return _getDeviceInfoPromise().then(function(json) {
            var guid = "";
            if (json.guid) {
                guid = json.guid;
            }
            var url = [normalizer("https://pay.video.qq.com/fcgi-bin/autopay?otype=json&platform="), platform, "&guid=", guid, "&versioncode=", getAppVersionBuild()].join('');
            return _autoPayPromise(url);
        });
    }

    function _getDeviceInfoPromise() {
        var $defer = $.Deferred();
        tenvideoReady(function() {
            TenvideoJSBridge.invoke("getDeviceInfo", null, function(json_msg) {
                var json = JSON.parse(json_msg);
                if (json.errCode == 0) {
                    $defer.resolve(json.result);
                } else {
                    $defer.reject("");
                }
            });
        });
        return $defer.promise();
    }

    function _autoPayPromise(url) {
        var $defer = $.Deferred();
        $.ajax({
            url: url,
            dataType: 'jsonp'
        }).done(function(json) {
            if (json && json.result.code == 0) {
                $defer.resolve(json);
            } else {
                $defer.reject();
            }
        }).fail(function(err) {
            $defer.reject(err);
        });
        return $defer.promise();
    }

    function isSandbox() {
        var _isSandbox = false;
        if (isAndroid && versionCompare(appVersion, "5.5.1")) {
            var sandboxParam = getUrlParam('sandbox');
            if (sandboxParam == 1) {
                _isSandbox = true;
            }
        }
        return _isSandbox;
    }

    function addTenvideoListener(listener, callback) {
        tenvideoReady(function() {
            TenvideoJSBridge.on(listener, function() {
                callback && callback();
            });
        });
    }

    module.exports = {
        tenvideoReady: tenvideoReady,
        getOfferId: getOfferId,
        getAppVersion: getAppVersion,
        getAppVersionBuild: getAppVersionBuild,
        versionCompare: versionCompare,
        getHlwPriceConfig: getHlwPriceConfig,
        getMainUserInfo: getMainUserInfo,
        isAccountLogin: isAccountLogin,
        getTicketNum: getTicketNum,
        getOrderNum: getOrderNum,
        getOrderNumNetwork: getOrderNumNetwork,
        getMainCookie: getMainCookie,
        getCookieParam: getCookieParam,
        getDeviceInfo: getDeviceInfo,
        getAppInfo: getAppInfo,
        getIapType: getIapType,
        isInreviewVer: isInreviewVer,
        getHlwDiscountAjax: getHlwDiscountAjax,
        getHlwDiscountDefault: getHlwDiscountDefault,
        getDefaultHlwParam: getDefaultHlwParam,
        getCouponCountAjax: getCouponCountAjax,
        getCouponPackageAjax: getCouponPackageAjax,
        getGtk: getGtk,
        getToken: getToken,
        getPlatform: getPlatform,
        isAndroidPf: isAndroidPf,
        isIosPf: isIosPf,
        getUin: getUin,
        getUserid: getUserid,
        getUrlParam: getUrlParam,
        refreshVipInfo: refreshVipInfo,
        showMessageButton: showMessageButton,
        openUrlCommon: openUrlCommon,
        getAutoPayParam: getAutoPayParam, //是否开启自动续费开关
        getSerialPayConf: getSerialPayConf,
        picSizeSet: picSizeSet,
        getCoverCorner: getCoverCorner,
        getParam: getParam,
        getUrlParam2: getUrlParam2,
        getAutoPayTag: getAutoPayTag, //管理自动续费
        isSandbox: isSandbox,
        addTenvideoListener: addTenvideoListener
    }; //exps;
});
